/*
 * $Id: canvasLine.c,v 1.2 2003/09/01 18:13:08 baum Exp $
 *
 * This file implements the line item of the canvas widget
 *
 * Copyright (c) 2001 - 2003 Peter G. Baum  http://www.dr-baum.net
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
   History:
        11: switched from GnoclWidgetOptions to GnoclOption
   2002-01: removed "smooth" and "splineSteps" options since they are not
            implemented in gnome-canvas-line.c
   2001-05: Begin of developement
 */

#include "canvas.h"
#include <string.h>
#include <assert.h>

static int optArrow( Tcl_Interp *interp, GnoclOption *opt, 
      GObject *obj, Tcl_Obj **ret );
static int optArrowShape( Tcl_Interp *interp, GnoclOption *opt, 
      GObject *obj, Tcl_Obj **ret );

static GnoclOption lineOptions[] = 
{
   { "-coords", GNOCL_LIST, NULL },
   { "-tags", GNOCL_LIST, NULL },
   { "-fill", GNOCL_OBJ, "fill_color_gdk", gnoclOptGdkColor },
   { "-arrow", GNOCL_OBJ, "", optArrow },
   { "-arrowShape", GNOCL_LIST, "", optArrowShape },
   { "-capStyle", GNOCL_OBJ, "cap_style", gnoclOptCapStyle },
   { "-joinStyle", GNOCL_OBJ, "join_style", gnoclOptJoinStyle },
   { "-width", GNOCL_DOUBLE, "width_units" },
   { "-pixelWidth", GNOCL_INT, "width_pixels" },
   { "-onMotion", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onEnter", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onLeave", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onButtonPress", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { "-onButtonRelease", GNOCL_OBJ, "", gnoclItemOptOnFunc },
   { NULL }
};

static const int coordsIdx = 0;
static const int tagsIdx   = 1;

static int optArrowShape( Tcl_Interp *interp, GnoclOption *opt, 
      GObject *obj, Tcl_Obj **ret )
{
   int k;
   double vals[3];

   if( Tcl_ListObjLength( interp, opt->val.obj, &k ) != TCL_OK
         || k != 3 )
   {
      Tcl_SetResult( interp, 
            "arrow shape must be a list of 3 double values", TCL_STATIC );
      return TCL_ERROR;
   }
   for( k = 0; k < 3; ++k )
   {
      Tcl_Obj *tp;
      if( Tcl_ListObjIndex( interp, opt->val.obj, k, &tp ) != TCL_OK 
         || Tcl_GetDoubleFromObj( interp, tp, &vals[k] ) != TCL_OK )
      {
         return TCL_ERROR;
      }
   }
   g_object_set( obj, "arrow_shape_a", vals[0], "arrow_shape_b", vals[1], 
         "arrow_shape_c", vals[2], NULL );
   return TCL_OK;
}

static int optArrow( Tcl_Interp *interp, GnoclOption *opt, 
      GObject *obj, Tcl_Obj **ret )
{
   const char *txt[] = { "none", "first", "last", "both", NULL };
   int idx;
   if( Tcl_GetIndexFromObj( interp, opt->val.obj, txt, "arrow",
         TCL_EXACT, &idx ) != TCL_OK )
      return TCL_ERROR;

   g_object_set( obj, 
            "first_arrowhead", (gboolean)(idx & 1), 
            "last_arrowhead", (gboolean)((idx & 2) != 0), NULL );
   return TCL_OK;
}

static int setOptions( Tcl_Interp *interp, Gnocl_CanvasItemInfo *info )
{
   gnoclResetSetOptions( lineOptions );

   if( gnoclSetOptions( interp, lineOptions, G_OBJECT( info->item ), -1 ) 
         != TCL_OK )
      return TCL_ERROR;

   if( gnoclCanvasHandleTags( interp, lineOptions + tagsIdx, info ) != TCL_OK )
      return TCL_ERROR;

   if( lineOptions[coordsIdx].status == GNOCL_STATUS_CHANGED )
   {
      GnomeCanvasPoints *points;
      int   noCoords;
      int   n;
      Tcl_Obj *coords = lineOptions[coordsIdx].val.obj;

      if( Tcl_ListObjLength( interp, coords, &noCoords ) != TCL_OK
            || noCoords < 4 || ( noCoords % 2 != 0 ) )
      {
         Tcl_SetResult( interp, "list-of-coordinates must contain an "
               "even numer of elements (at least 4)", TCL_STATIC );
         return TCL_ERROR;
      }
      points = gnome_canvas_points_new( noCoords / 2 );
      for( n = 0; n < noCoords; ++n )
      {
         Tcl_Obj *tp;
         if( Tcl_ListObjIndex( interp, coords, n, &tp ) != TCL_OK 
               || Tcl_GetDoubleFromObj( interp, tp, &points->coords[n] ) 
               != TCL_OK )
         {
            gnome_canvas_points_free( points );
            return TCL_ERROR;
         }
      }
      gnome_canvas_item_set( info->item, "points", points, NULL );
      gnome_canvas_points_free( points );
   }

   return TCL_OK;
}

static int parseOptions( Tcl_Interp *interp, int objc, 
      Tcl_Obj * const objv[], Gnocl_CanvasItemInfo *info )
{
   gnoclClearOptions( lineOptions );
   return gnoclParseOptions( interp, objc, objv, lineOptions );
}

static int clearOptions( Tcl_Interp *interp, Gnocl_CanvasItemInfo *info ) 
{
   gnoclClearOptions( lineOptions );
   return TCL_OK;
}

Gnocl_CanvasItemInfo *gnoclCanvasCreateLine( Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[], GnomeCanvasGroup *group )
{
   Gnocl_CanvasItemInfo *info = g_new( Gnocl_CanvasItemInfo, 1 );
   info->parseOptions = &parseOptions;
   info->setOptions = &setOptions;
   info->clearOptions = &clearOptions;
   info->command = NULL;
   info->item = gnome_canvas_item_new( group, 
         gnome_canvas_line_get_type( ), 
         "fill_color", "black", 
         "arrow_shape_a", 5., 
         "arrow_shape_b", 10., 
         "arrow_shape_c", 5.,
         NULL );

   /* init options */
   gnoclParseOptions( NULL, 0, NULL, lineOptions );

   return info;
}

